from textwrap import dedent

import pytest


@pytest.fixture
def dummy_module():
    return {
        'name': 'dummy',
        'stream': '0',
        'version': 1,
        'context': '2',
        'arch': 'noarch',
        'summary': 'Dummy Module',
        'description': 'One dummy module for your tests',
        'module_license': 'No License',
        'licenses': [],
        'package_nevras': [],
        'requires': {}
    }


@pytest.fixture
def dummy_module_mmd_as_string():
    return dedent("""
        ---
        document: modulemd
        version: 2
        data:
          name: dummy
          stream: 0
          version: 1
          context: 2
          summary: Dummy Module
          description: >-
            One dummy module for your tests
          license:
            module:
            - No License
          dependencies:
          - {}
        ...
    """).lstrip()



# -*- coding: utf-8 -*-

import argparse
import createrepo_c as cr
import gi
import logging
import os
import sys
import hawkey
from dnf.subject import Subject


gi.require_version('Modulemd', '2.0')
from gi.repository import Modulemd  # noqa: E402


DEFAULT_PROFILE = 'everything'


def parse_repodata(path):
    """
    Return a list of packages included in this repository
    """
    try:
        repomd = cr.Repomd(os.path.join(path, "repodata/repomd.xml"))
    except OSError as e:
        logging.error(e)
        exit(2)

    for record in repomd.records:
        if record.type == "primary":
            primary_xml_path = record.location_href

    def warningcb(warning_type, message):
        """Optional callback for warnings about
        wierd stuff and formatting in XML.
        :param warning_type: Integer value. One from
                             the XML_WARNING_* constants.
        :param message: String message.
        """
        logging.warning("PARSER WARNING: %s" % message)
        return True

    packages = []

    def pkgcb(pkg):
        # Called when whole package entry in xml is parsed
        packages.append(pkg)

    cr.xml_parse_primary(os.path.join(path, primary_xml_path),
                         pkgcb=pkgcb,
                         do_files=False,
                         warningcb=warningcb)

    return packages


def get_source_packages(packages):
    """
    Return the unique set of source package names
    """
    source_packages = set()
    for pkg in packages:
        # Get the source RPM NEVRA without the trailing ".rpm"
        subject = Subject(pkg.rpm_sourcerpm[:-4])

        # Now get just the source RPM name
        nevras = subject.get_nevra_possibilities(forms=[hawkey.FORM_NEVRA])
        for nevra in nevras:
            source_packages.add(nevra.name)

    return source_packages

def get_arg_parser():
    description = ("Generates modules.yaml file with a module, "
                   "that provides all RPM packages that are available "
                   "within a repository.")
    parser = argparse.ArgumentParser("repo2module", description=description)

    debug = parser.add_mutually_exclusive_group()
    debug.add_argument("--debug", action="store_true", default=False)
    debug.add_argument("--nodebug", action="store_false", dest="debug")

    parser.add_argument("-n", "--module-name",
                        default=os.path.basename(os.environ.get("PWD")),
                        help="Default is the current directory name")
    parser.add_argument("-s", "--module-stream", default="rolling")
    parser.add_argument("-v", "--module-version", default=1, type=int)
    parser.add_argument("-c", "--module-context", default="abcdef12")
    parser.add_argument("-O", "--to-stdout", default=False, action="store_true")
    parser.add_argument("repo_path")
    parser.add_argument("modules_yaml", default="modules.yaml", nargs="?")
    return parser


def cli():
    parser = get_arg_parser()
    args = parser.parse_args()

    if args.debug:
        logging.basicConfig(level=logging.DEBUG)

    if not os.path.isdir(args.repo_path):
        logging.error("No such directory: {0}".format(args.repo_path))
        exit(1)

    abs_repo_path = os.path.abspath(args.repo_path)
    if not args.to_stdout:
        abs_modules_yaml = os.path.abspath(args.modules_yaml)

    packages = parse_repodata(abs_repo_path)

    # Create module stream framework
    stream = Modulemd.ModuleStreamV2.new(args.module_name, args.module_stream)
    stream.set_version(args.module_version)
    stream.set_context(args.module_context)
    stream.set_summary('<auto-generated module summary>')
    stream.set_description('<auto-generated module description>')
    stream.add_module_license("MIT")
    stream.add_content_license("<FILL THIS IN>")

    source_packages = get_source_packages(packages)

    for srcpkg in source_packages:
        component = Modulemd.ComponentRpm.new(srcpkg)
        component.set_rationale('Present in the repository')
        stream.add_component(component)

    common_profile = Modulemd.Profile.new(DEFAULT_PROFILE)

    for pkg in packages:
        stream.add_rpm_artifact(pkg.nevra())
        stream.add_rpm_api(pkg.name)
        common_profile.add_rpm(pkg.name)

    stream.add_profile(common_profile)

    # Add defaults for this module
    defaults = Modulemd.DefaultsV1.new(args.module_name)
    defaults.set_default_stream(args.module_stream)
    defaults.add_default_profile_for_stream(args.module_stream, DEFAULT_PROFILE)

    index = Modulemd.ModuleIndex.new()
    index.add_module_stream(stream)
    index.add_defaults(defaults)

    if args.to_stdout:
        sys.stdout.write(index.dump_to_string())
        return

    logging.debug("Writing YAML to {}".format(abs_modules_yaml))
    try:
        with open(abs_modules_yaml, 'w') as output:
            output.write(index.dump_to_string())
    except PermissionError as e:
        logging.error("Could not write YAML to file: {}".format(e))
        exit(3)


if __name__ == "__main__":
    cli()



#!/usr/bin/python3

"""
A small wrapper around `createrepo_c` and `modifyrepo_c` to provide an easy tool
for generating module repositories.

This is supposed to be only a temporary solution, in the future we would like to
have the modularity support implemented in `createrepo_c` itself. See

https://bugzilla.redhat.com/show_bug.cgi?id=1816753

Please see the official Fedora Modularity documentation for the reference of how
module repositories should be created

https://docs.fedoraproject.org/en-US/modularity/hosting-modules/
"""


import os
import sys
import subprocess
import argparse
from distutils.version import LooseVersion

import gi
gi.require_version("Modulemd", "2.0")
from gi.repository import Modulemd  # noqa: E402


def run_createrepo(args):
    cmd = ["createrepo_c"] + args
    proc = subprocess.run(cmd, check=True)
    return proc.returncode


def run_modifyrepo(path, compress_type=None):
    cmd = [
        "modifyrepo_c",
        "--mdtype", "modules",
        os.path.join(path, "modules.yaml"),
        os.path.join(path, "repodata"),
    ]

    if compress_type:
        cmd.extend(["--compress-type", compress_type])

    proc = subprocess.run(cmd, check=True)
    return proc.returncode


def find_module_yamls(path):
    """
    Recursivelly find modulemd YAML files and return a list of their relative
    paths.
    """
    matches = []
    for root, dirnames, filenames in os.walk(path):
        for filename in filenames:
            if not filename.endswith((".yaml", ".yaml.gz")):
                continue
            filepath = os.path.join(root, filename)
            if not is_yaml_valid_modulemd(filepath):
                continue
            matches.append(filepath)
    return matches


def is_yaml_valid_modulemd(path):
    """
    Determine whether a YAML file is a valid modulemd file.
    If this simple check gets more and more complex, instead of adding
    logic here, consider calling `modulemd-validator` command, e.g.

        modulemd-validator -q foo.yaml

    """
    idx = Modulemd.ModuleIndex.new()
    (ret, _) = idx.update_from_file(path, strict=True)
    return ret


def dump_modules_yaml(path, yamls):
    """
    Go through all module YAMLs and merge them into one big YAML file.
    Then store the output as modules.yaml file in the `path` directory
    """
    cmd = ["modulemd-merge", "-i"]
    for yaml in yamls:
        cmd.append(yaml)
    cmd.append(os.path.join(path, "modules.yaml"))
    subprocess.run(cmd)


def createrepo_c_with_builtin_module_support():
    """
    There is a built-in support for module metadata in createrepo_c since
    version 0.16.1, please see the change log:
    rpm -q --changelog createrepo_c |less
    """
    cmd = ["rpm", "-q", "createrepo_c", "--queryformat", "%{VERSION}"]
    createrepo_c_version = subprocess.check_output(cmd).decode("utf-8")
    return LooseVersion(createrepo_c_version) >= LooseVersion("0.16.1")


def main():
    run_createrepo(sys.argv[1:])
    if createrepo_c_with_builtin_module_support():
        return

    parser = get_arg_parser()
    args, _ = parser.parse_known_args()
    yamls = find_module_yamls(args.path)
    if not yamls:
        return
    dump_modules_yaml(args.path, yamls)
    run_modifyrepo(args.path, "gz")


def get_arg_parser():
    # We are not going to define the whole parser here. Instead, we want to
    # pass all the input parameters to `createrepo_c` and let it handle them.
    #
    # We only need to define parser for a small subset of parameters that we
    # need to work within this script.

    description = ("A small wrapper around createrepo_c and modifyrepo_c to"
                   "provide an easy tool for generating module repositories")
    parser = argparse.ArgumentParser("createrepo_mod", description=description)
    parser.add_argument("path", metavar="directory_to_index",
                        help="Directory to index")
    return parser


if __name__ == "__main__":
    try:
        main()
    except subprocess.CalledProcessError as ex:
        sys.stderr.write("Error: {0}\n".format(str(ex)))
        sys.exit(1)



#!/usr/bin/python3
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

with open('requirements.txt') as f:
    requires = f.read().splitlines()

setup(
    name='modulemd_tools',
    version='0.1',
    packages=find_packages(exclude=("tests",)),
    url='https://github.com/rpm-software-management/modulemd-tools',
    license='MIT',
    author='Jakub Kadlcik',
    author_email='frostyx@email.cz',
    description='Helper lib for working with modulemd YAML definitions',
    long_description=long_description,
    long_description_content_type='text/markdown',
    # dir2module also requires libmodulemd and libdnf not available on PyPI
    # and must be installed separately.
    # On Fedora, this is done with `dnf install python3-libmodulemd python3-dnf`
    install_requires=requires,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux",
    ],
    include_package_data=True,
)



from textwrap import dedent

import pytest


@pytest.fixture
def moduleB_yaml():
    return dedent("""
        ---
        document: modulemd-defaults
        version: 1
        data:
          module: bar
          stream: stable
          profiles:
            stable: [everything]
        ...
        ---
        document: modulemd
        version: 2
        data:
          name: bar
          stream: stable
          version: 234
          context: fc33
          summary: Let's put some bar's summary here
          description: >-
            Let's put some bar's description here
          license:
            module:
            - Bar License
            content:
            - A fresh bar license for your test suites
          profiles:
            everything:
              rpms:
              - bar-utils
          api:
            rpms:
            - bar-utils
          components:
            rpms:
              bar-utils:
                rationale: Present in the repository
          artifacts:
            rpms:
            - bar-utils-2:3-4.fc33.noarch
        ...
    """).lstrip()


@pytest.fixture
def two_modules_merged_yamls():
    return dedent("""
        ---
        document: modulemd-defaults
        version: 1
        data:
          module: bar
          stream: stable
          profiles:
            stable: [everything]
        ...
        ---
        document: modulemd
        version: 2
        data:
          name: bar
          stream: stable
          version: 234
          context: fc33
          summary: Let's put some bar's summary here
          description: >-
            Let's put some bar's description here
          license:
            module:
            - Bar License
            content:
            - A fresh bar license for your test suites
          profiles:
            everything:
              rpms:
              - bar-utils
          api:
            rpms:
            - bar-utils
          components:
            rpms:
              bar-utils:
                rationale: Present in the repository
          artifacts:
            rpms:
            - bar-utils-2:3-4.fc33.noarch
        ...
        ---
        document: modulemd-defaults
        version: 1
        data:
          module: foo
          stream: devel
          profiles:
            devel: [everything]
        ...
        ---
        document: modulemd
        version: 2
        data:
          name: foo
          stream: devel
          version: 123
          context: fc33
          summary: Let's put some foo's summary here
          description: >-
            Let's put some foo's description here
          license:
            module:
            - Foo License
            content:
            - A fresh foo license for your test suites
          profiles:
            everything:
              rpms:
              - foo-utils
          api:
            rpms:
            - foo-utils
          components:
            rpms:
              foo-utils:
                rationale: Present in the repository
          artifacts:
            rpms:
            - foo-utils-1:2-3.fc33.noarch
        ...
    """).lstrip()


@pytest.fixture
def module_with_repodata_dir():
    return dedent("""
        ---
        document: modulemd-defaults
        version: 1
        data:
          module: dummy
          stream: rolling
          profiles:
            rolling: [everything]
        ...
        ---
        document: modulemd
        version: 2
        data:
          name: dummy
          stream: rolling
          version: 1
          context: abcdef12
          summary: <auto-generated module summary>
          description: >-
            <auto-generated module description>
          license:
            module:
            - MIT
            content:
            - <FILL THIS IN>
          profiles:
            everything:
              rpms:
              - python-django-bash-completion
          api:
            rpms:
            - python-django-bash-completion
          components:
            rpms:
              python-django:
                rationale: Present in the repository
          artifacts:
            rpms:
            - python-django-bash-completion-0:3.0.10-3.fc33.noarch
        ...
        ---
        document: modulemd-defaults
        version: 1
        data:
          module: foo
          stream: devel
          profiles:
            devel: [everything]
        ...
        ---
        document: modulemd
        version: 2
        data:
          name: foo
          stream: devel
          version: 123
          context: fc33
          summary: Let's put some foo's summary here
          description: >-
            Let's put some foo's description here
          license:
            module:
            - Foo License
            content:
            - A fresh foo license for your test suites
          profiles:
            everything:
              rpms:
              - foo-utils
          api:
            rpms:
            - foo-utils
          components:
            rpms:
              foo-utils:
                rationale: Present in the repository
          artifacts:
            rpms:
            - foo-utils-1:2-3.fc33.noarch
        ...
    """).lstrip()



#!/usr/bin/python3

"""
Recursively read RPMs from DIR or read them from specified pkglist
and generate N:S:V:C:A.modulemd.yaml for them.
"""

import argparse
import fnmatch
import os
import sys

import gi
import hawkey
import rpm
from dnf.subject import Subject


gi.require_version("Modulemd", "2.0")
from gi.repository import Modulemd  # noqa: E402


class Module(object):
    """
    Provide a high-level interface for representing modules and yaml generation
    based on their values.
    """
    def __init__(self, name, stream, version, context, arch, summary,
                 description, module_license, licenses, package_nevras, requires):
        self.name = name
        self.stream = stream
        self.version = version
        self.context = context
        self.arch = arch
        self.summary = summary
        self.description = description
        self.module_license = module_license
        self.licenses = licenses
        self.package_nevras = package_nevras
        self.requires = requires

    @property
    def filename(self):
        """
        Generate filename for a module yaml
        """
        return "{N}:{S}:{V}:{C}:{A}.modulemd.yaml".format(
            N=self.name, S=self.stream, V=self.version,
            C=self.context, A=self.arch)

    def dumps(self):
        """
        Generate modulemd yaml based on input parameters and return it as a string
        """
        mod_stream = Modulemd.ModuleStreamV2.new(self.name, self.stream)
        mod_stream.set_version(self.version)
        mod_stream.set_context(self.context)
        mod_stream.set_summary(self.summary)
        mod_stream.set_description(self.description)

        mod_stream.add_module_license(self.module_license)
        for pkglicense in self.licenses:
            mod_stream.add_content_license(pkglicense)

        for nevra in self.package_nevras:
            mod_stream.add_rpm_artifact(nevra)

        dependencies = Modulemd.Dependencies()
        for depname, depstream in self.requires.items():
            dependencies.add_runtime_stream(depname, depstream)
        mod_stream.add_dependencies(dependencies)

        index = Modulemd.ModuleIndex.new()
        index.add_module_stream(mod_stream)
        return index.dump_to_string()

    def dump(self):
        """
        Generate modulemd yaml based on input parameters write it into file
        """
        with open(self.filename, "w") as moduleyaml:
            moduleyaml.write(self.dumps())


class Package(object):
    """
    Provide a high-level package interface for the needs of module generation
    """

    def __init__(self, path):
        self.path = path
        self.header = self._get_header()

    @property
    def nevra(self):
        """
        Takes a package filename and returns its NEVRA
        """
        filename = os.path.basename(self.path)
        if not filename.endswith(".rpm"):
            raise ValueError("File name doesn't end with '.rpm': {}".format(self.path))

        # @TODO: construct NEVRA from rpm header
        subject = Subject(filename)
        nevras = subject.get_nevra_possibilities(forms=[hawkey.FORM_NEVRA])
        for nevra in nevras:
            # For some reason `nevra.release` contains also the architecture
            return "{N}-{E}:{V}-{RA}".format(N=nevra.name, E=nevra.epoch or 0,
                                             V=nevra.version, RA=nevra.release)

    @property
    def license(self):
        """
        Examine a RPM package and return its license
        """
        return self.header["license"]

    @property
    def has_modularity_label(self):
        """
        Examine a RPM package and see if it has `ModularityLabel` set in its header
        """
        return bool(self.header["modularitylabel"])

    def _get_header(self):
        """
        Examine a RPM package file and return its header
        See docs.fedoraproject.org/en-US/Fedora_Draft_Documentation/0.1/html/RPM_Guide/ch16s04.html
        """
        ts = rpm.TransactionSet()
        ts.setKeyring(rpm.keyring())
        ts.setVSFlags(rpm._RPMVSF_NOSIGNATURES | rpm._RPMVSF_NODIGESTS)
        with open(self.path, "r") as f:
            hdr = ts.hdrFromFdno(f.fileno())
            return hdr


def find_packages(path):
    """
    Recursively find RPM packages in a `path` and return their list
    """
    packages = []
    for root, _, filenames in os.walk(path):
        for filename in fnmatch.filter(filenames, "*.rpm"):
            if filename.endswith(".src.rpm"):
                continue
            packages.append(os.path.join(root, filename))
    return packages


def find_packages_in_file(path):
    """
    Parse a text file containing a list of packages and return their list
    """
    with open(path, "r") as pkglist:
        return pkglist.read().splitlines()


def parse_nsvca(nsvca):
    """
    Take module name, stream, version, context and architecture in a N:S:V:C:A
    format and return them as a separate values.
    """
    split = nsvca.split(":")
    if len(split) != 5:
        raise ValueError("N:S:V:C:A in unexpected format")
    split[2] = int(split[2])
    return split


def get_arg_parser():
    description = (
        "Recursively read RPMs from DIR or read them from specified pkglist."
        "If any RPM is missing on unreadable, error out."
        "Populate artifacts/rpms with RPM NEVRAs."
        "Populate license/content with list of RPM licenses."

        "Write N:S:V:C:A.modulemd.yaml in the current directory."
        "Make sure the yaml is in modulemd v2 format."
    )
    parser = argparse.ArgumentParser("dir2module", description=description)
    parser.add_argument("nsvca",
                        help=("Module name, stream version, context and "
                              "architecture in a N:S:V:C:A format"))
    parser.add_argument("-m", "--summary", required=True, help="Module summary")
    parser.add_argument("-d", "--description", help="Module description")
    parser.add_argument("-l", "--license", default="MIT", help="Module license")
    parser.add_argument("-r", "--requires", action="append",
                        help=("Module runtime dependencies in a N:S format. "
                              "For multiple dependencies, repeat this option"))
    parser.add_argument("--force", action="store_true",
                        help="Suppress all constraints and hope for the best")
    parser.add_argument("--stdout", action="store_true",
                        help=("By defult the output is saved in a file. Use "
                              "this to suppress it and print to the STDOUT"))

    input_group = parser.add_mutually_exclusive_group(required=True)
    input_group.add_argument("--dir", help="")
    input_group.add_argument("--pkglist", help="")
    return parser


def parse_dependencies(deps):
    if deps is None:
        return {}
    return dict([dep.split(":") for dep in deps])


def main():
    parser = get_arg_parser()
    args = parser.parse_args()
    name, stream, version, context, arch = parse_nsvca(args.nsvca)

    if args.dir:
        path = os.path.expanduser(args.dir)
        packages = find_packages(path)
    else:
        path = os.path.expanduser(args.pkglist)
        packages = find_packages_in_file(path)

    packages = [Package(package) for package in packages]
    licenses = {package.license for package in packages}
    nevras = {package.nevra for package in packages}

    requires = parse_dependencies(args.requires)
    description = args.description \
        or "This module has been generated using {0} tool".format(parser.prog)

    missing_labels = []
    for package in packages:
        if not package.has_modularity_label:
            missing_labels.append(package.path)
            msg = "ERROR: " if args.force else "WARNING: "
            msg += "RPM does not have `modularitylabel` header set: {}".format(package.path)
            print(msg)

    if missing_labels and not args.force:
        raise RuntimeError("All packages need to contain the `modularitylabel` header. "
                           "To suppress this constraint, use `--force` parameter")

    module = Module(name, stream, version, context, arch, args.summary,
                    description, args.license, licenses, nevras, requires)

    if args.stdout:
        print(module.dumps())
    else:
        module.dump()
        print("Created {0}".format(module.filename))


if __name__ == "__main__":
    try:
        main()
    except (RuntimeError, ValueError) as ex:
        sys.stderr.write("Error: {0}\n".format(str(ex)))
        sys.exit(1)



from textwrap import dedent

import pytest


@pytest.fixture
def module_yaml_output():
    return dedent("""
        ---
        document: modulemd-defaults
        version: 1
        data:
          module: dummy
          stream: rolling
          profiles:
            rolling: [everything]
        ...
        ---
        document: modulemd
        version: 2
        data:
          name: dummy
          stream: rolling
          version: 1
          context: abcdef12
          summary: <auto-generated module summary>
          description: >-
            <auto-generated module description>
          license:
            module:
            - MIT
            content:
            - <FILL THIS IN>
          profiles:
            everything:
              rpms:
              - python-django-bash-completion
          api:
            rpms:
            - python-django-bash-completion
          components:
            rpms:
              python-django:
                rationale: Present in the repository
          artifacts:
            rpms:
            - python-django-bash-completion-0:3.0.10-3.fc33.noarch
        ...
    """).lstrip()



#!/usr/bin/python3
# -*- coding: utf-8 -*-
import os.path

from setuptools import setup


dirname = os.path.dirname(os.path.realpath(__file__))


with open(os.path.join(dirname, "README.md"), "r") as fh:
    long_description = fh.read()

with open(os.path.join(dirname, 'requirements.txt')) as f:
    requires = f.read().splitlines()

setup(
    name='repo2module',
    version='0.1',
    packages=['repo2module'],
    url='https://github.com/rpm-software-management/modulemd-tools',
    license='MIT',
    author='Stephen Gallagher',
    author_email='sgallagh@redhat.com',
    description='Tool to generate module metadata for a yum repo',
    long_description=long_description,
    long_description_content_type='text/markdown',
    # repo2module also requires libmodulemd, libdnf and createrepo_c which are
    # not available on PyPI and must be installed separately. On Fedora, this
    # is done with
    # `dnf install python3-libmodulemd python3-createrepo_c python3-dnf`
    install_requires=requires,
    entry_points={
        'console_scripts': [
            'repo2module=repo2module.cli:cli'],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux",
    ],
    include_package_data=True,
)



#!/usr/bin/python3
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

with open('requirements.txt') as f:
    requires = f.read().splitlines()

setup(
    name='modulemd-merge',
    version='0.1',
    packages=find_packages(),
    url='https://github.com/rpm-software-management/modulemd-tools',
    license='MIT',
    author='Gerd v. Egidy',
    author_email='gerd.von.egidy@intra2net.com',
    description='Merge several modules.yaml files (rpm modularity metadata) into one',
    long_description=long_description,
    long_description_content_type='text/markdown',
    # modulemd-merge also requires libmodulemd and createrepo_c which are
    # not available on PyPI and must be installed separately.
    # On Fedora, this is done with `dnf install python3-libmodulemd python3-createrepo_c`
    install_requires=requires,
    entry_points={
        'console_scripts': [
            'modulemd-merge=modulemd_merge.modulemd_merge:main'],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux",
    ],
    include_package_data=True,
)



#!/usr/bin/python3
#
# merge several modules.yaml files (rpm modularity metadata) into one
#
# Copyright (c) 2020 Gerd v. Egidy
# License: MIT
# https://github.com/rpm-software-management/modulemd-tools
#

import os
import sys
import logging
import argparse

import createrepo_c as cr

import gi
gi.require_version("Modulemd", "2.0")
from gi.repository import Modulemd  # noqa: E402


def hande_repomd(args, merger, repomd_filename):
    try:
        repomd = cr.Repomd(repomd_filename)
    except (RuntimeError, ValueError) as err:
        if not args.ignore_no_input:
            raise err
        logging.debug("{}: error loading repomd.xml: {}".format(repomd_filename, str(err)))
        return False

    # repomd was loaded and decoded successfully
    modules_path = False
    for record in repomd.records:
        if record.type == "modules":
            modules_path = record.location_href

    if not modules_path:
        logging.debug("{fn}: no modules section found in repomd.xml".format(fn=repomd_filename))
        if not args.ignore_no_input:
            raise ValueError('{fn} does not contain a modules section'.format(fn=repomd_filename))
        return False

    # strip repodata-prefix-dir from location_href
    filename = os.path.join(os.path.dirname(repomd_filename), os.path.basename(modules_path))
    if os.path.isfile(filename):
        return merge_file(merger, filename)

    filename = os.path.join(os.path.dirname(repomd_filename), "../", modules_path)
    if os.path.isfile(filename):
        return merge_file(merger, filename)

    logging.debug("{fn}: modules section found in repomd.xml, but href file {href} does not exist".
                  format(fn=repomd_filename, href=filename))
    if not args.ignore_no_input:
        raise ValueError("{fn}: modules section found in repomd.xml, but href file {href} does "
                         "not exist".format(fn=repomd_filename, href=filename))
    return False


# check type and existence of input file, directly or indirectly calls merge_file
# when a yaml is found
def merge_input(args, merger, filename):
    if os.path.isdir(filename):
        logging.debug("{}: is a directory".format(filename))
        if os.path.isfile(os.path.join(filename, "repodata/repomd.xml")):
            hande_repomd(args, merger, os.path.join(filename, "repodata/repomd.xml"))
        elif os.path.isfile(os.path.join(filename, "repomd.xml")):
            hande_repomd(args, merger, os.path.join(filename, "repomd.xml"))
        else:
            logging.debug("{}: no repomd.xml in or below directory".format(filename))
            if not args.ignore_no_input:
                raise ValueError('{fn} is a directory, but no repomd.xml file existing in or below'
                                 .format(fn=filename))

    elif os.path.isfile(filename):
        logging.debug("{}: is a regular file".format(filename))

        if os.path.basename(filename) == "repomd.xml":
            hande_repomd(args, merger, filename)
        else:
            merge_file(merger, filename)

    else:
        logging.debug("{}: file does not exist".format(filename))
        if not args.ignore_no_input:
            raise ValueError('input file {fn} does not exist'.format(fn=filename))


def merge_file(merger, filename):
    logging.debug("{}: Loading YAML".format(filename))

    index = Modulemd.ModuleIndex.new()
    ret, failures = index.update_from_file(filename, True)
    if not ret:
        failmsglst = []
        for err in failures:
            failmsglst.append(err.get_gerror().message)
        raise ValueError('Error parsing {fn}'.format(fn=filename), failmsglst)

    modnames = index.get_module_names()
    defstreams = index.get_default_streams()
    logging.info("{}: Found {} modulemds and {} modulemd-defaults".format(filename, len(modnames),
                 len(defstreams)))

    merger.associate_index(index, 0)


def get_arg_parser():
    description = "Merge several modules.yaml files (rpm modularity metadata) into one."
    parser = argparse.ArgumentParser("modulemd-merge", description=description,
                                     formatter_class=argparse.RawTextHelpFormatter)

    # flag options
    parser.add_argument("-v", "--verbose", help="increase output verbosity",
                        action="store_true")
    parser.add_argument("-d", "--debug", help="debug output verbosity",
                        action="store_true")
    parser.add_argument("-i", "--ignore-no-input", help="ignore non-existing input files",
                        action="store_true")
    parser.add_argument("-O", "--to-stdout", help="print YAML output to stdout",
                        action="store_true")

    # positional arguments
    parser.add_argument("input", nargs="+", help="input filename(s) or directories.\n"
                        "repomd.xml files are parsed and modules hrefs contained are merged.\n"
                        "If a directory is given, it is searched for repodata/repomd.xml\n"
                        "and repomd.xml")
    parser.add_argument("output", help="YAML output filename")
    return parser


def parse_args():
    parser = get_arg_parser()
    args = parser.parse_args()

    if args.verbose and not args.debug:
        logging.basicConfig(format='%(message)s', level=logging.INFO)
    if args.debug:
        logging.basicConfig(format='%(message)s', level=logging.DEBUG)

    return args


def main():
    args = parse_args()

    merger = Modulemd.ModuleIndexMerger.new()

    for i in args.input:
        merge_input(args, merger, i)

    merged_index = merger.resolve()

    modnames = merged_index.get_module_names()
    defstreams = merged_index.get_default_streams()
    logging.info("merged result: {} modulemds and {} modulemd-defaults".format(len(modnames),
                                                                               len(defstreams)))

    if args.to_stdout:
        output = sys.stdout
    else:
        logging.debug("Writing YAML to {}".format(args.output))
        output = open(args.output, 'w')

    if len(modnames) == 0 and len(defstreams) == 0:
        # properly writing a completely empty yaml document
        logging.debug("Writing an empty YAML")
        output.write("")
    else:
        output.write(merged_index.dump_to_string())

    if not args.to_stdout:
        output.close()


if __name__ == "__main__":
    try:
        main()
    except (RuntimeError, ValueError) as ex:
        sys.stderr.write("Error: {0}\n".format(str(ex)))
        sys.exit(1)



#!/usr/bin/python3

"""
Generate module-build-macros SRPM package
See https://github.com/rpm-software-management/modulemd-tools/issues/10
See https://pagure.io/fm-orchestrator/issue/1217
"""

import os
import sys
import contextlib
import locale
import datetime
import textwrap
import tempfile
import glob
import logging
import subprocess
import argparse
from modulemd_tools.yaml import _yaml2stream, load


log = logging.getLogger("MBS")


def generate_module_build_macros_spec(mmd, disttag, td, conflicts_file=None):
    name = "module-build-macros"
    version = "0.1"
    release = "1"
    with set_locale(locale.LC_TIME, "C"):
        today = datetime.date.today().strftime("%a %b %d %Y")

    conflicts = ""
    if conflicts_file:
        with open(conflicts_file, "r") as f:
            conflicts = f.read()

    spec_content = textwrap.dedent("""
        %global dist {disttag}
        %global modularitylabel {module_name}:{module_stream}:{module_version}:{module_context}
        %global _module_name {module_name}
        %global _module_stream {module_stream}
        %global _module_version {module_version}
        %global _module_context {module_context}

        Name:       {name}
        Version:    {version}
        Release:    {release}%dist
        Summary:    Package containing macros required to build generic module
        BuildArch:  noarch

        Group:      System Environment/Base
        License:    MIT
        URL:        http://fedoraproject.org

        Source1:    macros.modules

        {filter_conflicts}

        %description
        This package is used for building modules with a different dist tag.
        It provides a file /usr/lib/rpm/macros.d/macro.modules and gets read
        after macro.dist, thus overwriting macros of macro.dist like %%dist
        It should NEVER be installed on any system as it will really mess up
        updates, builds, ....


        %build

        %install
        mkdir -p %buildroot/etc/rpm 2>/dev/null |:
        cp %SOURCE1 %buildroot/etc/rpm/macros.zz-modules
        chmod 644 %buildroot/etc/rpm/macros.zz-modules


        %files
        /etc/rpm/macros.zz-modules



        %changelog
        * {today} Fedora-Modularity - {version}-{release}{disttag}
        - autogenerated macro by Module Build Service (MBS)
    """).format(
        disttag=disttag,
        today=today,
        name=name,
        version=version,
        release=release,
        module_name=mmd.get_module_name(),
        module_stream=mmd.get_stream_name(),
        module_version=mmd.get_version(),
        module_context=mmd.get_context(),
        filter_conflicts=conflicts,
    )
    fd = open(os.path.join(td, "%s.spec" % name), "w")
    fd.write(spec_content)
    fd.close()


def generate_module_build_macros_source(mmd, disttag, td):
    modulemd_macros = ""
    buildopts = mmd.get_buildopts()
    if buildopts:
        modulemd_macros = buildopts.get_rpm_macros() or ""

    macros_content = textwrap.dedent("""
        # General macros set by MBS

        %dist {disttag}
        %modularitylabel {module_name}:{module_stream}:{module_version}:{module_context}
        %_module_build 1
        %_module_name {module_name}
        %_module_stream {module_stream}
        %_module_version {module_version}
        %_module_context {module_context}

        # Macros set by module author:

        {modulemd_macros}
    """).format(
        disttag=disttag,
        module_name=mmd.get_module_name(),
        module_stream=mmd.get_stream_name(),
        module_version=mmd.get_version(),
        module_context=mmd.get_context(),
        modulemd_macros=modulemd_macros,
    )
    sources_dir = os.path.join(td, "SOURCES")
    os.mkdir(sources_dir)
    fd = open(os.path.join(sources_dir, "macros.modules"), "w")
    fd.write(macros_content)
    fd.close()


def generate_module_build_macros_srpm(mmd, disttag, td, conflicts_file=None):
    name = "module-build-macros"
    sources_dir = os.path.join(td, "SOURCES")

    generate_module_build_macros_spec(mmd, disttag, td, conflicts_file)
    generate_module_build_macros_source(mmd, disttag, td)

    log.debug("Building %s.spec" % name)

    # We are not interested in the rpmbuild stdout...
    null_fd = open(os.devnull, "w")
    execute_cmd(
        [
            "rpmbuild",
            "-bs",
            "%s.spec" % name,
            "--define",
            "_topdir %s" % td,
            "--define",
            "_sourcedir %s" % sources_dir,
        ],
        cwd=td,
        stdout=null_fd,
    )
    null_fd.close()
    sdir = os.path.join(td, "SRPMS")
    srpm_paths = glob.glob("%s/*.src.rpm" % sdir)
    assert len(srpm_paths) == 1, "Expected exactly 1 srpm in %s. Got %s" % (sdir, srpm_paths)

    log.debug("Wrote srpm into %s" % srpm_paths[0])
    return srpm_paths[0]


# ------------------------------------------------------------------------------
# Copy-pasted functions from https://pagure.io/fm-orchestrator
# @TODO Figure out what to do with it

@contextlib.contextmanager
def set_locale(*args, **kwargs):
    saved = locale.setlocale(locale.LC_ALL)
    yield locale.setlocale(*args, **kwargs)
    locale.setlocale(locale.LC_ALL, saved)


def execute_cmd(args, stdout=None, stderr=None, cwd=None):
    """
    Executes command defined by `args`. If `stdout` or `stderr` is set to
    Python file object, the stderr/stdout output is redirecter to that file.
    If `cwd` is set, current working directory is set accordingly for the
    executed command.

    :param args: list defining the command to execute.
    :param stdout: Python file object to redirect the stdout to.
    :param stderr: Python file object to redirect the stderr to.
    :param cwd: string defining the current working directory for command.
    :raises RuntimeError: Raised when command exits with non-zero exit code.
    """
    out_log_msg = ""
    if stdout and hasattr(stdout, "name"):
        out_log_msg += ", stdout log: %s" % stdout.name
    if stderr and hasattr(stderr, "name"):
        out_log_msg += ", stderr log: %s" % stderr.name

    log.info("Executing the command \"%s\"%s" % (" ".join(args), out_log_msg))
    proc = subprocess.Popen(args, stdout=stdout, stderr=stderr, cwd=cwd)
    out, err = proc.communicate()

    if proc.returncode != 0:
        err_msg = "Command '%s' returned non-zero value %d%s" % (args, proc.returncode, out_log_msg)
        raise RuntimeError(err_msg)
    return out, err


# ------------------------------------------------------------------------------

def get_arg_parser():
    name = "modulemd-generate-macros"
    description = ("Generate `module-build-macros` SRPM package, which is a "
                   "central piece for building modules. It should be present "
                   "in the buildroot before any other module packages are "
                   "submitted to be built.")
    parser = argparse.ArgumentParser(name, description=description)
    parser.add_argument(
        "yaml",
        help="Path to modulemd YAML file "
    )
    parser.add_argument(
        "--disttag",
        required=False,
        help="Disttag"
    )
    parser.add_argument(
        "--conflicts-from-file",
        required=False,
        help=("Path to a file containing conflicts definitions and their "
              "reasoning. Content of this file gets simply pasted into the "
              "specfile")
    )
    return parser


def main():
    parser = get_arg_parser()
    args = parser.parse_args()

    disttag = args.disttag
    if not disttag:
        disttag = "%{?dist}"

    try:
        mmd = _yaml2stream(load(args.yaml))
    except RuntimeError as ex:
        print("Failed to parse {}".format(args.yaml))
        print("Make sure it is a valid modulemd YAML file\n")
        print("Hint: {}".format(str(ex)))
        sys.exit(1)

    try:
        td = tempfile.mkdtemp(prefix="module_build_service-build-macros")
        srpm = generate_module_build_macros_srpm(
            mmd, disttag, td, conflicts_file=args.conflicts_from_file)
        print(srpm)
    except RuntimeError as ex:
        print("\nFailed to generate module-build-macros SRPM package\n")
        print("{0}\n".format(str(ex)))
        print("Please review {}/module-build-macros.spec".format(td))
        sys.exit(1)


if __name__ == "__main__":
    main()



import unittest
import os.path

from dir2module.dir2module import Module, Package, find_packages, parse_nsvca, parse_dependencies


dirname = os.path.dirname(os.path.realpath(__file__))
test_packages_dir = os.path.join(dirname, "packages")


def test_dummy_module_loading(dummy_module):
    m = Module(**dummy_module)
    assert isinstance(m, Module)


def test_dummy_module_filename(dummy_module):
    m = Module(**dummy_module)
    assert m.filename == "dummy:0:1:2:noarch.modulemd.yaml"


@unittest.skip("Does not work with the latest libmodulemd (2.12.0)")
def test_dummy_module_dumps(dummy_module, dummy_module_mmd_as_string):
    m = Module(**dummy_module)
    output = m.dumps()
    assert output == dummy_module_mmd_as_string


def test_normal_package_loading():
    p = Package(os.path.join(test_packages_dir,
                             "python-django-bash-completion-3.0.10-3.fc33.noarch.rpm"))
    assert isinstance(p, Package)


def test_normal_package_license():
    p = Package(os.path.join(test_packages_dir,
                             "python-django-bash-completion-3.0.10-3.fc33.noarch.rpm"))
    assert p.license == "BSD"


def test_normal_package_has_not_modularity_label():
    p = Package(os.path.join(test_packages_dir,
                             "python-django-bash-completion-3.0.10-3.fc33.noarch.rpm"))
    assert p.has_modularity_label is False


def test_modular_package_has_modularity_label():
    p = Package(os.path.join(
        test_packages_dir,
        "python-django-bash-completion-1.6.11.8-1.module_f33+9570+f65235c8.noarch.rpm"))
    assert p.has_modularity_label is True


@unittest.skip("Does not work with the latest libmodulemd (2.12.0)")
def test_find_packages_in_directory():
    packages_files = [os.path.basename(rpm_abs_path)
                      for rpm_abs_path in find_packages(test_packages_dir)]
    assert packages_files == [
        'python-django-bash-completion-3.0.10-3.fc33.noarch.rpm',
        'python-django-bash-completion-1.6.11.8-1.module_f33+9570+f65235c8.noarch.rpm',
    ]


def test_parse_nsvca():
    assert parse_nsvca("dummy:0:1:2:noarch") == ['dummy', '0', 1, '2', 'noarch']


def test_parse_dependencies():
    assert parse_dependencies(['dummy:0']) == {'dummy': '0'}



import glob
import logging
import os.path
import shutil

import pytest

from createrepo_mod.createrepo_mod import (
    run_createrepo, run_modifyrepo, find_module_yamls, dump_modules_yaml)


logger = logging.getLogger(__name__)

dirname = os.path.dirname(os.path.realpath(__file__))
test_packages_dir = os.path.join(dirname, "packages")
test_module_yamls_dir = os.path.join(dirname, "module_yamls")


def test_run_createrepo(test_output_dir):
    # I'd rather use shutil.copytree with dirs_exist_ok=True, but it requires Python 3.8+
    for package in glob.glob(os.path.join(test_packages_dir, "*.rpm")):
        shutil.copy(package, test_output_dir)
    retval = run_createrepo([test_output_dir])
    assert os.path.isdir(os.path.join(test_output_dir, "repodata"))
    assert os.path.isfile(os.path.join(test_output_dir, "repodata", "repomd.xml"))
    assert retval == 0


def test_find_module_yamls():
    assert len(find_module_yamls(test_module_yamls_dir)) > 0


@pytest.mark.skipif(shutil.which("modulemd-merge") is None, reason="requires modulemd-merge")
def test_dump_modules_yaml(test_output_dir):
    dump_modules_yaml(test_output_dir, find_module_yamls(test_module_yamls_dir))
    assert os.path.isfile(os.path.join(test_output_dir, "modules.yaml"))


def test_run_modifyrepo(test_output_dir):
    if not os.path.isfile(os.path.join(test_output_dir, "modules.yaml")):
        logger.info("Seems like test_dump_modules_yaml was skipped. "
                    "Creating modules.yaml from dummy.yaml.")
        shutil.copy(os.path.join(test_module_yamls_dir, "dummy.yaml"),
                    os.path.join(test_output_dir, "modules.yaml"))

    assert os.path.isfile(os.path.join(test_output_dir, "modules.yaml"))
    retval = run_modifyrepo(test_output_dir)
    assert glob.glob(os.path.join(test_output_dir, "repodata", "*-modules.yaml.gz"))
    assert retval == 0



"""
Module for working with modulemd YAML definitions with the least abstractions as
possible. Within this module, modulemd YAMLs are represented simply as a string,
and all transformation functions are `str` -> `str`.
"""

import os
import gi
import yaml

gi.require_version("Modulemd", "2.0")
from gi.repository import Modulemd  # noqa: E402


def is_valid(mod_yaml):
    """
    Determine whether the `mod_yaml` string is a valid modulemd YAML definition
    """
    idx = Modulemd.ModuleIndex.new()
    try:
        ret, _ = idx.update_from_string(mod_yaml, strict=True)
        return ret
    except gi.repository.GLib.GError:
        return False


def validate(mod_yaml):
    """
    Same as `is_valid` but raises exception if the YAML is not valid.
    """
    idx = Modulemd.ModuleIndex.new()
    try:
        ret, failures = idx.update_from_string(mod_yaml, strict=True)
    except gi.repository.GLib.GError as ex:
        raise RuntimeError(ex)
    if not ret:
        raise RuntimeError(failures[0].get_gerror().message)
    return ret


def create(name, stream):
    """
    Create a minimal modulemd YAML definition containing only a module name and
    module stream. To set any additional attributes, use `update` function.
    """
    mod_stream = Modulemd.ModuleStreamV2.new(name, stream)
    mod_stream.set_summary("")
    mod_stream.set_description("")
    mod_stream.add_module_license("")
    index = Modulemd.ModuleIndex.new()
    index.add_module_stream(mod_stream)
    return index.dump_to_string()


def update(mod_yaml, name=None, stream=None, version=None, context=None,
           arch=None, summary=None, description=None, module_licenses=None,
           content_licenses=None, rpms_nevras=None, requires=None,
           buildrequires=None, api=None, filters=None, profiles=None,
           components=None):
    """
    Transform a given modulemd YAML string into another, updated one. The input
    string remains unchanged.

    This function allows to modify specified modulemd attributes while leaving
    the rest of them as is. For structured attributes, such as `module_licenses`
    which value is a list, new values are not appended to a list, but the new
    value is used instead.

    For the official documentation of the modulemd YAML format and it's values,
    please see
    https://github.com/fedora-modularity/libmodulemd/blob/main/yaml_specs/modulemd_stream_v2.yaml
    It will allow you to better understand the parameters of this function.

    Args:
        mod_yaml (str): An input modulelmd YAML
        name (str): The name of the module
        stream (str): Module update stream name
        version (int): Module version, integer, cannot be negative
        context (str): Module context flag
        arch (str): Module artifact architecture
        summary (str): A short summary describing the module
        description (str): A verbose description of the module
        module_licenses (list): A list of module licenses
        content_licenses (list): A list of licenses used by the packages in
            the module.
        rpms_nevras (list): RPM artifacts shipped with this module
        requires (dict): Module runtime dependencies represented as a `dict` of
            module names as keys and list of streams as their values.
        buildrequires (dict): Module buildtime dependencies represented as a
            `dict` of module names as keys and list of streams as their values.
        api (list): The module's public RPM-level API represented as a list of
            package names.
        filters (list): Module component filters represented as a list of pckage
            names.
        profiles (dict): A `dict` of profile names as keys and lists of package
            names as their values.
        components (list): Functional components of the module represented as a
            `dict` with package names as keys and `dict`s representing the
            particular components as keys. The component `dict` should contain
            keys like `name`, `rationale`, `repository`, etc.

    Returns:
        An updated modulemd YAML represented as string

    """
    mod_stream = _yaml2stream(mod_yaml)

    # AFAIK It is not possible to change name and stream of an existing module,
    # so we need to workaround it by creating a new module with desired N:S and
    # then update it from the previous yaml
    name = name or mod_stream.get_module_name()
    stream = stream or mod_stream.get_stream_name()
    mod_stream = Modulemd.ModuleStreamV2.new(name, stream)
    mod_stream = mod_stream.read_string(mod_yaml, True, name, stream)

    if version:
        mod_stream.set_version(version)

    if context:
        mod_stream.set_context(context)

    if arch:
        mod_stream.set_arch(arch)

    if summary:
        mod_stream.set_summary(summary)

    if description:
        mod_stream.set_description(description)

    if module_licenses:
        mod_stream.clear_module_licenses()
        for module_license in module_licenses:
            mod_stream.add_module_license(module_license)

    if content_licenses:
        mod_stream.clear_content_licenses()
        for content_license in content_licenses:
            mod_stream.add_content_license(content_license)

    if rpms_nevras:
        mod_stream.clear_rpm_artifacts()
        for nevra in rpms_nevras:
            mod_stream.add_rpm_artifact(nevra)

    if api:
        mod_stream.clear_rpm_api()
        for rpm in api:
            mod_stream.add_rpm_api(rpm)

    if filters:
        mod_stream.clear_rpm_filters()
        for rpm in filters:
            mod_stream.add_rpm_filter(rpm)

    if profiles:
        mod_stream.clear_profiles()
        for profile_name, rpms in profiles.items():
            profile = Modulemd.Profile.new(profile_name)
            for rpm in rpms:
                profile.add_rpm(rpm)
            mod_stream.add_profile(profile)

    if components:
        mod_stream.clear_rpm_components()
        for component in components:
            component_rpm = Modulemd.ComponentRpm.new(component.pop("name"))
            for key, value in component.items():
                component_rpm.set_property(key, value)
            mod_stream.add_component(component_rpm)

    # Updating dependencies is quite messy because AFAIK the only operations
    # that `libmodoulemd` allows us to do is adding a runtime/buildtime
    # dependencies one be one and dropping all of them at once.
    # We need to help ourselves a little and drop all runtime dependencies and
    # re-populate them with the old ones if a new ones weren't set. Similarly
    # for buildrequires.
    old_deps = Modulemd.Dependencies()

    # Module can contain multiple pairs of dependencies. If we want to update
    # both `requires` and `buildrequires` at the same time, we can drop all
    # current dependencies and just set a new one. If we want to update only
    # one of them, we are getting to an ambiguous situation, not knowing what
    # pair of dependencies we should update. Let's just raise an exception.
    if (len(mod_stream.get_dependencies()) > 1
            and (requires or buildrequires)
            and not (requires and buildrequires)):
        raise AttributeError("Provided YAML contains multiple pairs of "
                             "dependencies. It is ambiguous which one to "
                             "update.")

    if mod_stream.get_dependencies():
        old_deps = mod_stream.get_dependencies()[0]
    new_deps = Modulemd.Dependencies()
    if requires:
        for depname, depstreams in requires.items():
            for depstream in depstreams:
                new_deps.add_runtime_stream(depname, depstream)
    else:
        for depname in old_deps.get_runtime_modules():
            for depstream in old_deps.get_runtime_streams(depname):
                new_deps.add_runtime_stream(depname, depstream)

    if buildrequires:
        for depname, depstreams in buildrequires.items():
            for depstream in depstreams:
                new_deps.add_buildtime_stream(depname, depstream)
    else:
        for depname in old_deps.get_buildtime_modules():
            for depstream in old_deps.get_buildtime_streams(depname):
                new_deps.add_buildtime_stream(depname, depstream)

    if requires or buildrequires:
        mod_stream.clear_dependencies()
        mod_stream.add_dependencies(new_deps)

    idx2 = Modulemd.ModuleIndex.new()
    idx2.add_module_stream(mod_stream)
    return idx2.dump_to_string()


def upgrade(mod_yaml, version):
    """
    Upgrade the input module YAML from an older version to a newer one.
    Downgrades aren't supported even in case where it would be possible.
    """
    parsed = yaml.safe_load(mod_yaml or "")
    if not parsed or "version" not in parsed:
        raise ValueError("Missing modulemd version")

    supported = [1, 2]
    if parsed["version"] not in supported or version not in supported:
        raise ValueError("Unexpected modulemd version")

    if parsed["version"] > version:
        raise ValueError("Cannot downgrade modulemd version")

    mod_stream = Modulemd.ModuleStream.read_string(
        mod_yaml,
        True,
        parsed["data"].get("name", ""),
        parsed["data"].get("stream", ""))
    mod_stream_upgraded = mod_stream.upgrade(version)
    return _stream2yaml(mod_stream_upgraded)


def load(path):
    """
    Load modulemd YAML definition from a file
    """
    with open(path, "r") as yaml_file:
        mod_yaml = yaml_file.read()
    validate(mod_yaml)
    return mod_yaml


def dump(mod_yaml, dest=None):
    """
    Dump modulemd YAML definition into a file

    If `dest` is not specified, the file will be created in the current working
    directory and it's name is going to be generated from the module attributes
    in the `N:S:V:C:A.modulemd.yaml` format.

    If `dest` is a directory, then the file is going to be generated in that
    directory.

    If `dest` is pointing to a (nonexisting) file, it is going to be dumped in
    its place.
    """
    filename = _generate_filename(mod_yaml)

    path = dest
    if not path:
        path = os.path.join(os.getcwd(), filename)
    elif os.path.isdir(path):
        path = os.path.join(dest, filename)

    with open(path, "w") as moduleyaml:
        moduleyaml.write(mod_yaml)


def _generate_filename(mod_yaml):
    """
    Generate filename for a module yaml
    """
    mod_stream = _yaml2stream(mod_yaml)
    return "{N}:{S}:{V}:{C}:{A}.modulemd.yaml".format(
        N=mod_stream.get_module_name(),
        S=mod_stream.get_stream_name(),
        V=mod_stream.get_version(),
        C=mod_stream.get_context(),
        A=mod_stream.get_arch() or "noarch")


def _yaml2stream(mod_yaml):
    try:
        return Modulemd.ModuleStream.read_string(mod_yaml, True, None, None)
    except gi.repository.GLib.GError as ex:
        raise ValueError(ex.message)


def _stream2yaml(mod_stream):
    idx = Modulemd.ModuleIndex.new()
    idx.add_module_stream(mod_stream)
    try:
        return idx.dump_to_string()
    except gi.repository.GLib.GError as ex:
        raise RuntimeError(ex.message)



import argparse
import os.path
from unittest.mock import patch

import pytest
import yaml

from modulemd_merge import modulemd_merge

dirname = os.path.dirname(os.path.realpath(__file__))
test_data_dir = os.path.join(dirname, "testdata")
test_repodata_dir = os.path.join(dirname, "testdata", "repodata")
test_repomd_file = os.path.join(dirname, "testdata", "repodata", "repomd.xml")


def _testrun_args(inputs, **kwargs):
    args = {'verbose': True, 'debug': True, 'input': inputs, 'to_stdout': True,
            'ignore_no_input': False}
    args.update(kwargs)
    return argparse.Namespace(**args)


def test_modulemd_merge_loading():
    assert modulemd_merge


def test_modulemd_merge_two_yamls(capsys, two_modules_merged_yamls):

    inputs = (os.path.join(test_data_dir, "moduleA.yaml"),
              os.path.join(test_data_dir, "moduleB.yaml"))
    with patch("argparse.ArgumentParser.parse_args", return_value=_testrun_args(inputs)):
        modulemd_merge.main()

    captured = capsys.readouterr()
    assert [d for d in yaml.load_all(captured.out, Loader=yaml.SafeLoader)] == [
        d for d in yaml.load_all(two_modules_merged_yamls, Loader=yaml.SafeLoader)]


def test_modulemd_merge_two_yamls_first_missing(capsys):

    inputs = (os.path.join(test_data_dir, "missing-file.yaml"),
              os.path.join(test_data_dir, "moduleB.yaml"))
    with patch("argparse.ArgumentParser.parse_args", return_value=_testrun_args(inputs)):
        with pytest.raises(ValueError) as excinfo:
            modulemd_merge.main()

    assert f"input file {test_data_dir}/missing-file.yaml does not exist" in str(excinfo.value)


def test_modulemd_merge_two_yamls_first_missing_ignore_no_input(capsys, moduleB_yaml):

    inputs = (os.path.join(test_data_dir, "missing-file.yaml"),
              os.path.join(test_data_dir, "moduleB.yaml"))
    kwargs = {"ignore_no_input": True}
    with patch("argparse.ArgumentParser.parse_args", return_value=_testrun_args(inputs, **kwargs)):
        modulemd_merge.main()

    captured = capsys.readouterr()
    assert [d for d in yaml.load_all(captured.out, Loader=yaml.SafeLoader)] == [
        d for d in yaml.load_all(moduleB_yaml, Loader=yaml.SafeLoader)]


def test_modulemd_merge_module_with_repodata_dir(capsys, module_with_repodata_dir):

    inputs = (os.path.join(test_data_dir, "moduleA.yaml"),
              test_repodata_dir)
    with patch("argparse.ArgumentParser.parse_args", return_value=_testrun_args(inputs)):
        modulemd_merge.main()

    captured = capsys.readouterr()
    assert [d for d in yaml.load_all(captured.out, Loader=yaml.SafeLoader)] == [
        d for d in yaml.load_all(module_with_repodata_dir, Loader=yaml.SafeLoader)]


def test_modulemd_merge_module_with_repomd_file(capsys, module_with_repodata_dir):

    inputs = (os.path.join(test_data_dir, "moduleA.yaml"),
              test_repomd_file)
    with patch("argparse.ArgumentParser.parse_args", return_value=_testrun_args(inputs)):
        modulemd_merge.main()

    captured = capsys.readouterr()
    assert [d for d in yaml.load_all(captured.out, Loader=yaml.SafeLoader)] == [
        d for d in yaml.load_all(module_with_repodata_dir, Loader=yaml.SafeLoader)]



import unittest
import os.path
import createrepo_c
import repo2module.cli
from repo2module.cli import parse_repodata, get_source_packages


dirname = os.path.dirname(os.path.realpath(__file__))
test_repo_dir = os.path.join(dirname, "rpmrepo")


def test_cli_module_loading():
    """Just a very simple/basic test that CLI module loading works."""
    assert repo2module.cli


def test_parse_repodata():
    packages = parse_repodata(test_repo_dir)
    assert len(packages) > 0
    assert all(isinstance(pkg, createrepo_c.Package) for pkg in packages)


def test_get_source_packages():
    packages = parse_repodata(test_repo_dir)
    source_packages = get_source_packages(packages)
    assert source_packages == {'python-django'}


@unittest.skip("Does not work with the latest libmodulemd (2.12.0)")
def test_repo2module(module_yaml_output):
    # runner = CliRunner()
    # result = runner.invoke(repo2module.cli.cli, ['-n', 'dummy', '-O', test_repo_dir])
    # assert result.exit_code == 0
    # assert result.output == module_yaml_output
    raise NotImplementedError



import os
import unittest
from unittest import mock
import yaml
from distutils.version import LooseVersion
from modulemd_tools.yaml import (is_valid, validate, create, update, dump,
                                 upgrade, _yaml2stream, _stream2yaml)

import gi
gi.require_version("Modulemd", "2.0")
from gi.repository import Modulemd  # noqa: E402


def old_libmodulemd():
    """
    Reading YAML string via `Modulemd.ModuleStream.read_string` and dumping it
    again encapsulates its value in double-quotes, and it messes up with some of
    our tests (since the older version does exactly the opposite). Let's just
    skip those few test on EPEL8 until it receives an update.
    See also `080e2bb`
    """
    return LooseVersion(Modulemd.get_version()) < LooseVersion("2.11.1")


class TestYaml(unittest.TestCase):

    def test_is_valid(self):
        self.assertTrue(is_valid(yaml1))
        self.assertFalse(is_valid(yaml2_invalid))

    def test_is_valid_wrong_format(self):
        self.assertFalse(is_valid("this is not yaml"))

    def test_validate(self):
        self.assertTrue(validate(yaml1))
        with self.assertRaises(Exception) as context:
            validate(yaml2_invalid)
        self.assertIn("The module and stream names are required",
                      str(context.exception))

    def test_validate_wrong_format(self):
        with self.assertRaises(RuntimeError) as context:
            validate("this is not yaml")
        self.assertIsInstance(context.exception, RuntimeError)
        self.assertIn("Unexpected YAML event in document stream",
                      str(context.exception))

    def test_create(self):
        result = create("foo", "stable")
        mod1 = yaml.safe_load(result)
        self.assertEqual(mod1["document"], "modulemd")
        self.assertEqual(mod1["version"], 2)
        self.assertEqual(mod1["data"]["name"], "foo")
        self.assertEqual(mod1["data"]["stream"], "stable")
        self.assertEqual(mod1["data"]["summary"], None)
        self.assertEqual(mod1["data"]["description"], "")
        self.assertEqual(mod1["data"]["license"]["module"], [None])

    def test_update_after_build(self):
        """
        This is an example modulemd transformation that copr-backend would do
        after finishing a module build.
        """
        # First, let's check that we can parse the modulemd YAML definition and
        # that it looks as expected
        mod1 = yaml.safe_load(yaml1)
        self.assertNotIn("arch", mod1["data"])
        for rpm in mod1["data"]["artifacts"]["rpms"]:
            self.assertTrue(rpm.startswith("LTK-"))

        # When copr-backend finishes a module build, it needs to update the
        # architecture and built artifacts
        rpms = ["foo-0:1.2-1.fc32.x86_64", "bar-1:1.2-3.fc32.x86_64"]
        result = update(yaml1, arch="x86_64", rpms_nevras=rpms)

        # Now let's see
        mod2 = yaml.safe_load(result)
        self.assertEqual(mod2["data"]["arch"], "x86_64")
        self.assertEqual(set(mod2["data"]["artifacts"]["rpms"]), set(rpms))

    def test_update_modulemd_creation(self):
        """
        This is an example modulemd transformation that copr-frontend would do
        when generating a new module via web UI (not exactly, but it would
        change these properties).
        """
        mod1 = yaml.safe_load(yaml1)
        self.assertEqual(mod1["data"]["version"], 123)
        self.assertEqual(mod1["data"]["context"], "f32")
        self.assertEqual(mod1["data"]["summary"], "Summary and stuff")
        self.assertEqual(mod1["data"]["description"],
                         "This module has been generated using dir2module tool")
        self.assertEqual(mod1["data"]["license"]["module"], ["MIT"])
        self.assertEqual(mod1["data"]["license"]["content"],
                         ["BSD", "GPLv2", "MIT"])

        result = update(yaml1, version=234, context="f33", summary="A new sum",
                        description="A new desc", module_licenses=["BSD"],
                        content_licenses=["MIT", "WTFPL"])

        mod1 = yaml.safe_load(result)
        self.assertEqual(mod1["data"]["version"], 234)
        self.assertEqual(mod1["data"]["context"], "f33")
        self.assertEqual(mod1["data"]["summary"], "A new sum")
        self.assertEqual(mod1["data"]["description"], "A new desc")
        self.assertEqual(set(mod1["data"]["license"]["module"]), {"BSD"})
        self.assertEqual(set(mod1["data"]["license"]["content"]),
                         {"MIT", "WTFPL"})

    def test_update_modulemd_nsvca(self):
        """
        It is not expected change module name or stream, not even `libmodulemd`
        classes allow it. In case you want to update a module name, you need to
        create a new one. I say f*ck it, let us change whatever we want.
        """
        mod1 = yaml.safe_load(yaml1)
        self.assertEqual(mod1["data"]["name"], "foo")
        self.assertEqual(mod1["data"]["stream"], "devel")
        self.assertEqual(mod1["data"]["version"], 123)
        self.assertEqual(mod1["data"]["context"], "f32")

        result = update(yaml1, name="bar", stream="stable",
                        version=234, context="f33")

        mod2 = yaml.safe_load(result)
        self.assertEqual(mod2["data"]["name"], "bar")
        self.assertEqual(mod2["data"]["stream"], "stable")
        self.assertEqual(mod2["data"]["version"], 234)
        self.assertEqual(mod2["data"]["context"], "f33")

        self.assertEqual(mod2["data"]["summary"], "Summary and stuff")
        self.assertEqual(mod1["data"]["license"]["module"], ["MIT"])

    def test_update_modulemd_dependencies(self):
        mod1 = yaml.safe_load(yaml1)
        deps1 = mod1["data"]["dependencies"][0]
        self.assertEqual(deps1["requires"], {"platform": ["f32"]})
        self.assertEqual(deps1["buildrequires"], {"platform": ["f32"]})

        result = update(yaml1, requires={"foo": ["f33"]},
                        buildrequires={"bar": ["master"]})

        mod2 = yaml.safe_load(result)
        deps2 = mod2["data"]["dependencies"][0]
        self.assertEqual(deps2["requires"], {"foo": ["f33"]})
        self.assertEqual(deps2["buildrequires"], {"bar": ["master"]})

    def test_update_modulemd_runtime_dependencies(self):
        """
        Dropping dependencies can be tricky so should rather make sure that if
        we want to update only runtime dependencies, the buildtime dependencies
        remain untouched.
        """
        mod1 = yaml.safe_load(yaml1)
        deps1 = mod1["data"]["dependencies"][0]
        self.assertEqual(deps1["requires"], {"platform": ["f32"]})
        self.assertEqual(deps1["buildrequires"], {"platform": ["f32"]})

        result = update(yaml1, requires={"foo": ["f33"]})

        mod2 = yaml.safe_load(result)
        deps2 = mod2["data"]["dependencies"][0]
        self.assertEqual(deps2["requires"], {"foo": ["f33"]})
        self.assertEqual(deps2["buildrequires"], {"platform": ["f32"]})

    def test_update_without_dependencies(self):
        """
        The logic for updating dependencies is a bit complicated and can fail
        when dependencies are not present in the modulemd YAML at all.
        """
        validate(yaml3_no_deps)
        mod1 = yaml.safe_load(yaml3_no_deps)
        self.assertNotIn("dependencies", mod1["data"])

        result = update(yaml3_no_deps, summary="Updated summary")
        mod2 = yaml.safe_load(result)
        self.assertEqual(mod2["data"]["summary"], "Updated summary")
        self.assertNotIn("dependencies", ["data"])

        result = update(yaml3_no_deps, requires={"foo": ["bar"]})
        mod3 = yaml.safe_load(result)
        self.assertEqual(mod3["data"]["dependencies"][0]["requires"],
                         {"foo": ["bar"]})

    def test_update_modulemd_with_multiple_pairs_of_deps(self):
        """
        While uncommon, it's not impossible for there to be more than one
        Dependencies object in the list.
        """
        validate(yaml6_multiple_pairs_of_deps)
        mod1 = yaml.safe_load(yaml6_multiple_pairs_of_deps)
        self.assertEqual(len(mod1["data"]["dependencies"]), 2)

        dependencies = [
            {"buildrequires": {"platform": ["-epel8"]},
             "requires": {"platform": ["-epel8"]}},
            {"buildrequires": {"libfoo": ["rolling"],
                               "platform": ["epel8"]},
             "requires": {"libfoo": ["rolling"],
                          "platform": ["epel8"]}},
        ]
        self.assertEqual(mod1["data"]["dependencies"], dependencies)

        requires = {"foo": ["bar"]}
        with self.assertRaises(AttributeError) as context:
            update(yaml6_multiple_pairs_of_deps, requires=requires)
        self.assertIn("Provided YAML contains multiple pairs of dependencies. "
                      "It is ambiguous which one to update.",
                      str(context.exception))

        buildrequires = {"baz": ["qux"]}
        with self.assertRaises(AttributeError) as context:
            update(yaml6_multiple_pairs_of_deps, buildrequires=buildrequires)
        self.assertIn("Provided YAML contains multiple pairs of dependencies. "
                      "It is ambiguous which one to update.",
                      str(context.exception))

        result = update(yaml6_multiple_pairs_of_deps,
                        requires=requires, buildrequires=buildrequires)
        mod2 = yaml.safe_load(result)
        self.assertEqual(len(mod2["data"]["dependencies"]), 1)
        self.assertEqual(mod2["data"]["dependencies"][0], {
            "requires": {"foo": ["bar"]},
            "buildrequires": {"baz": ["qux"]},
        })

    def test_update_modulemd_api(self):
        mod1 = yaml.safe_load(yaml1)
        self.assertNotIn("api", mod1["data"])

        result = update(yaml1, api=["foo", "bar"])
        mod2 = yaml.safe_load(result)
        self.assertEqual(set(mod2["data"]["api"]["rpms"]), {"foo", "bar"})

        result = update(result, api=["baz"])
        mod3 = yaml.safe_load(result)
        self.assertEqual(set(mod3["data"]["api"]["rpms"]), {"baz"})

    def test_update_modulemd_filters(self):
        mod1 = yaml.safe_load(yaml1)
        self.assertNotIn("filter", mod1["data"])

        result = update(yaml1, filters=["foo", "bar"])
        mod2 = yaml.safe_load(result)
        self.assertEqual(set(mod2["data"]["filter"]["rpms"]), {"foo", "bar"})

        result = update(result, filters=["baz"])
        mod3 = yaml.safe_load(result)
        self.assertEqual(set(mod3["data"]["filter"]["rpms"]), {"baz"})

    def test_update_modulemd_profiles(self):
        mod1 = yaml.safe_load(yaml1)
        self.assertNotIn("profiles", mod1["data"])

        profiles = {
            "default": ["foo"],
            "ubercool": ["foo", "bar"]
        }
        result = update(yaml1, profiles=profiles)
        mod2 = yaml.safe_load(result)
        self.assertEqual(len(mod2["data"]["profiles"]), 2)
        self.assertEqual(set(mod2["data"]["profiles"]["default"]["rpms"]),
                         {"foo"})
        self.assertEqual(set(mod2["data"]["profiles"]["ubercool"]["rpms"]),
                         {"foo", "bar"})

        result = update(result, profiles={"minimal": ["baz"]})
        mod3 = yaml.safe_load(result)
        self.assertEqual(len(mod3["data"]["profiles"]), 1)
        self.assertEqual(set(mod3["data"]["profiles"]["minimal"]["rpms"]),
                         {"baz"})

    def test_update_modulemd_components(self):
        mod1 = yaml.safe_load(yaml1)
        self.assertNotIn("components", mod1["data"])

        components = [
            {"name": "foo", "rationale": "Just because", "buildorder": 10},
            {"name": "bar", "rationale": "It's a dep"},
        ]
        result = update(yaml1, components=components)
        mod2 = yaml.safe_load(result)
        self.assertEqual(len(mod2["data"]["components"]["rpms"]), 2)
        self.assertEqual(mod2["data"]["components"]["rpms"]["foo"], {
            "rationale": "Just because",
            "buildorder": 10,
        })
        self.assertEqual(mod2["data"]["components"]["rpms"]["bar"], {
            "rationale": "It's a dep",
        })

        components = [{
            "name": "baz",
            "rationale": "Testing component properties",
            "buildorder": 20,
            "ref": "master",
            "repository": "http://foo.bar/baz.git",
        }]
        result = update(result, components=components)
        mod2 = yaml.safe_load(result)
        self.assertEqual(len(mod2["data"]["components"]["rpms"]), 1)
        baz = mod2["data"]["components"]["rpms"]["baz"]
        self.assertEqual(baz, {
            "rationale": "Testing component properties",
            "buildorder": 20,
            "ref": "master",
            "repository": "http://foo.bar/baz.git",
        })

    def test_upgrade(self):
        result = upgrade(yaml4_v1, 2)
        mod_stream = _yaml2stream(result)
        self.assertEqual(mod_stream.get_mdversion(), 2)
        self.assertEqual(mod_stream.get_module_name(), "")
        self.assertEqual(mod_stream.get_summary(),
                         "A test module in all its beautiful beauty")

    @unittest.skipIf(old_libmodulemd(), "Old modulemd drops stream value quotes")
    def test_upgrade_to_same_version(self):
        result = upgrade(yaml1, 2)
        self.assertEqual(result, yaml1)

    def test_upgrade_cannot_downgrade(self):
        with self.assertRaises(ValueError) as context:
            upgrade(yaml1, 1)
        self.assertIn("Cannot downgrade modulemd version",
                      str(context.exception))

    def test_upgrade_empty_yaml(self):
        for mod_yaml in [None, "", "foo: bar"]:
            with self.assertRaises(ValueError) as context:
                upgrade(mod_yaml, 2)
            self.assertIn("Missing modulemd version", str(context.exception))

    def test_upgrade_unexpected_version(self):
        # Neither current modulemd version cannot be unexpected
        with self.assertRaises(ValueError) as context:
            upgrade("version: 5", 2)
        self.assertIn("Unexpected modulemd version", str(context.exception))

        # Nor the wanted one
        with self.assertRaises(ValueError) as context:
            upgrade("version: 1", 5)
        self.assertIn("Unexpected modulemd version", str(context.exception))

    def test_dump(self):
        with mock.patch("builtins.open") as mock_open:
            dump(yaml1)
            mock_open.assert_called_once()
            path = os.path.join(os.getcwd(),
                                "foo:devel:123:f32:noarch.modulemd.yaml")
            self.assertEqual(mock_open.call_args[0][0], path)

        with mock.patch("builtins.open") as mock_open:
            dump(yaml1, dest="/tmp")
            mock_open.assert_called_once()
            path = "/tmp/foo:devel:123:f32:noarch.modulemd.yaml"
            self.assertEqual(mock_open.call_args[0][0], path)

        with mock.patch("builtins.open") as mock_open:
            dump(yaml1, dest="/tmp/testmodule.yaml")
            mock_open.assert_called_once()
            self.assertEqual(mock_open.call_args[0][0], "/tmp/testmodule.yaml")

    def test_yaml2stream(self):
        mod_stream = _yaml2stream(yaml1)
        self.assertEqual(mod_stream.get_module_name(), "foo")
        self.assertEqual(mod_stream.get_summary(), "Summary and stuff")

        with self.assertRaises(ValueError) as context:
            _yaml2stream("")
        self.assertIn("YAML didn't begin with STREAM_START.",
                      str(context.exception))

        with self.assertRaises(ValueError) as context:
            _yaml2stream(yaml5_multiple_streams)
        self.assertIn("YAML contained more than a single subdocument",
                      str(context.exception))

        with self.assertRaises(ValueError) as context:
            _yaml2stream(yaml5_multiple_modules)
        self.assertIn("YAML contained more than a single subdocument",
                      str(context.exception))

    @unittest.skipIf(old_libmodulemd(), "Old modulemd drops stream value quotes")
    def test_stream2yaml(self):
        mod_stream = _yaml2stream(yaml1)
        self.assertEqual(_stream2yaml(mod_stream), yaml1)

        mod_stream.set_summary(None)
        with self.assertRaises(RuntimeError) as context:
            _stream2yaml(mod_stream)
        self.assertIn("Could not validate stream to emit: Summary is missing",
                      str(context.exception))


yaml1 = """---
document: modulemd
version: 2
data:
  name: foo
  stream: "devel"
  version: 123
  context: f32
  summary: Summary and stuff
  description: >-
    This module has been generated using dir2module tool
  license:
    module:
    - MIT
    content:
    - BSD
    - GPLv2
    - MIT
  dependencies:
  - buildrequires:
      platform: [f32]
    requires:
      platform: [f32]
  artifacts:
    rpms:
    - LTK-0:1.5.0-8.fc32.x86_64
    - LTK-debuginfo-0:1.5.0-8.fc32.x86_64
    - LTK-debugsource-0:1.5.0-8.fc32.x86_64
...
"""


# Invalid because of missing name
yaml2_invalid = """
---
document: modulemd
version: 2
data:
  stream: "devel"
  version: 123
  context: f32
  summary: Summary and stuff
  description: >-
    This module has been generated using dir2module tool
  license:
    module:
    - MIT
    content:
    - BSD
    - GPLv2
    - MIT
  dependencies:
  - {}
  artifacts:
    rpms:
    - LTK-0:1.5.0-8.fc32.x86_64
    - LTK-debuginfo-0:1.5.0-8.fc32.x86_64
    - LTK-debugsource-0:1.5.0-8.fc32.x86_64
...
"""


# Doesn't have any buildrequires nor requires, still a valid modulemd though
yaml3_no_deps = """
---
document: modulemd
version: 2
data:
  name: foo
  stream: "devel"
  version: 123
  context: f32
  summary: Summary and stuff
  description: >-
    This module has been generated using dir2module tool
  license:
    module:
    - MIT
    content:
    - BSD
    - GPLv2
    - MIT
  artifacts:
    rpms:
    - LTK-0:1.5.0-8.fc32.x86_64
    - LTK-debuginfo-0:1.5.0-8.fc32.x86_64
    - LTK-debugsource-0:1.5.0-8.fc32.x86_64
...
"""


# This is a module in modulemd v1 format, taken from
# https://src.fedoraproject.org/modules/testmodule/raw/fancy/f/testmodule.yaml
# I just cropped the description a bit.
yaml4_v1 = """
document: modulemd
version: 1
data:
    summary: A test module in all its beautiful beauty
    description: This module demonstrates how to write simple modulemd files
    license:
        module: [ MIT ]
    dependencies:
        buildrequires:
            platform: f33
        requires:
            platform: f33
    references:
        community: https://fedoraproject.org/wiki/Modularity
        documentation: https://fedoraproject.org/wiki/Fedora_Packaging_Guidelines_for_Modules
        tracker: https://taiga.fedorainfracloud.org/project/modularity
    profiles:
        default:
            rpms:
                - tangerine
    api:
        rpms:
            - perl-Tangerine
            - tangerine
    components:
        rpms:
            perl-List-Compare:
                rationale: A dependency of tangerine.
                ref: master
        modules:
            dwm:
                rationale: Testing module inclusion.
                ref: 6.1
                buildorder: 10
"""


# A merged YAML containing multiple streams of one module
yaml5_multiple_streams = """
---
document: modulemd
version: 2
data:
  name: foo
  stream: "master"
  version: 1
  summary: A test module in all its beautiful beauty
  description: >-
    Some description
  license:
    module:
    - MIT
...
---
document: modulemd
version: 2
data:
  name: foo
  stream: "stable"
  version: 123
  context: f32
  summary: Summary and stuff
  description: >-
    This module has been generated using dir2module tool
  license:
    module:
    - MIT
...
"""


# A merged YAML containing multiple modules
yaml5_multiple_modules = """
---
document: modulemd
version: 2
data:
  name: foo
  stream: "master"
  version: 1
  summary: A test module in all its beautiful beauty
  description: >-
    Some description
  license:
    module:
    - MIT
...
---
document: modulemd
version: 2
data:
  name: bar
  stream: "stable"
  version: 123
  context: f32
  summary: Summary and stuff
  description: >-
    This module has been generated using dir2module tool
  license:
    module:
    - MIT
...
"""


# A module with multiple pairs of dependencies
yaml6_multiple_pairs_of_deps = """
---
document: modulemd
version: 2
data:
  name: foo
  stream: "master"
  version: 1
  summary: A test module in all its beautiful beauty
  description: >-
    Some description
  license:
    module:
    - MIT
  dependencies:
  - buildrequires:
      platform: [ -epel8 ]
    requires:
      platform: [ -epel8 ]
  - buildrequires:
      platform: [ epel8 ]
      libfoo: [ rolling ]
    requires:
      platform: [ epel8 ]
      libfoo: [ rolling ]
...
"""


if __name__ == '__main__':
    unittest.main()
